#-----------------------------------------------------------------------------
require 'sketchup.rb'
require 'layers_panel/Win32API.so'
#-----------------------------------------------------------------------------

# Sketchup.send_action(CMD_RUBY_CONSOLE)

module JBB_LayersPanel


#-----------------------------------------------------------------------------
	WS_CAPTION  = 0x00C00000
	WS_EX_TOOLWINDOW = 0x00000080

	GWL_STYLE   = -16
	GWL_EXSTYLE = -20

	# SetWindowPos() flags
	SWP_NOSIZE       = 0x0001
	SWP_NOMOVE       = 0x0002
	SWP_DRAWFRAME    = 0x0020
	SWP_FRAMECHANGED = 0x0020
	SWP_NOREPOSITION = 0x0200
	WS_MAXIMIZEBOX =  0x10000
	WS_MINIMIZEBOX =  0x20000
	WS_SIZEBOX     =  0x40000

	# Windows Functions
	#FindWindow    = Win32API.new("user32.dll" , "FindWindow"   , 'PP' , 'L')
	#FindWindowEx  = Win32API.new("user32.dll", "FindWindowEx" , 'LLPP', 'L')
	SetWindowPos  = Win32API.new("user32.dll" , "SetWindowPos" , 'LLIIIII', 'I')
	SetWindowLong = Win32API.new("user32.dll" , "SetWindowLong", 'LIL', 'L')
	GetWindowLong = Win32API.new("user32.dll" , "GetWindowLong", 'LI' , 'L')
	GetActiveWindow = Win32API.new("user32.dll", "GetActiveWindow", '', 'L')
	#GetForegroundWindow = Win32API.new("user32.dll", "GetForegroundWindow", '', 'L')
	GetWindowText = Win32API.new("user32.dll", "GetWindowText", 'LPI', 'I')
	GetWindowTextLength = Win32API.new("user32.dll", "GetWindowTextLength", 'L', 'I')
#-----------------------------------------------------------------------------
	
	LAYER_IS_HIDDEN_ON_NEW_PAGES = 0x0020
	LAYER_HIDDEN_BY_DEFAULT = 0x0001
	
	@lpversion = "0.3.1"
	@model = Sketchup.active_model
	@layers = @model.layers
	@layerDictID = nil
	@dialog = nil
	@allowSerialize = true
	@previousPageDict = nil
	@previousPageDict2 = nil
	@previousPageDict3 = nil
	@check = nil
	
	class << self
		attr_accessor :model, :layers, :layerDictID, :dialog, :allowSerialize, :previousPageDict, :previousPageDict2, :previousPageDict3, :check
	end
	
	
	### LAYER SERIALIZE ### ------------------------------------------------------
	
	def self.incLayerDictID
		@model.start_operation("Layers Panel", false, false, true)
		@layerDictID = @layerDictID + 1
		@model.set_attribute("jbb_layerspanel", "layerDictID", @layerDictID) #Store incremented layerDictID in model attribute dict
		# puts "incLayerDictID"
		@model.commit_operation
	end#def
	
	def self.initializeLayerDictID
		if @layerDictID == nil
			@model.start_operation("Initialize Layers Panel", false, false, true)
			if @model.get_attribute("jbb_layerspanel", "layerDictID") != nil #Get layerDictID from model if exists
				@layerDictID = @model.get_attribute("jbb_layerspanel", "layerDictID")
			else #Else, create it
				@layers[0].set_attribute("jbb_layerspanel", "ID", 0) #Give Layer0 ID 0
				@layerDictID = 0
			end#if
			self.incLayerDictID
			@model.commit_operation 
			# puts "Current layerDictID : " + @layerDictID.to_s
		end#if
	end#def
	
	def self.IdLayer(layer) #Give a unique custom id to a layer
		if layer.get_attribute("jbb_layerspanel", "ID") != nil 
			#puts layer.name + " already IDed " + layer.get_attribute("jbb_layerspanel", "ID").to_s
		else
			@model.start_operation("ID layer", false, false, true)
			layer.set_attribute("jbb_layerspanel", "ID", @layerDictID)
			# puts "layerDictID " + @layerDictID.to_s
			self.incLayerDictID
			@model.commit_operation 
		end#if
	end#def
	
	
	### LAYER DELETE METHOD ### ------------------------------------------------------
	#Simple modification of TIG's snippet delete-layer.rb
	#Basically, move or delete layer content, then creates an entry for every layer except one to delete, then purge layers, then delete entries
	
	def self.deleteLayer(layer, delete_geometry=false)
		@model.start_operation("Delete layer")
		@allowSerialize = false
		ents=@model.entities; defs=@model.definitions
		if delete_geometry
			allents=[]
			@model.entities.each{|e|allents<<e if e.valid? and e.respond_to?(:layer)and e.layer==layer}
			@model.definitions.each{|d|d.entities.each{|e|allents<<e if e.valid? and e.respond_to?(:layer)and e.layer==layer}}
			allents.each{|e|e.erase! if e.valid?}
		else ### move geom to Layer0 etc
			@model.entities.each{|e|e.layer=nil if e.respond_to?(:layer)and e.layer==layer}
			@model.definitions.each{|d|d.entities.each{|e|e.layer=nil if e.respond_to?(:layer)and e.layer==layer}}
		end#if
		group=@model.entities.add_group();gents=group.entities ### temporarily use other layers
		temp=gents.add_group()
		temp.layer=nil
		(@layers.to_a-[layer]).each{|layer|tc=temp.copy;tc.layer=layer}
		@model.active_layer=nil if @model.active_layer==layer ### ensure layer is not current layer
		@layers.purge_unused ### purge layer from browser
		group.erase! ### erase! the temporary layer user, use set as was.
		@allowSerialize = true
		@dialog.execute_script("storeSerialize();")
		@model.commit_operation 
	end#def
  
  
  
	### WEBDIALOG & CALLBACKS ### ------------------------------------------------------
	
	# Create the WebDialog instance
	@dialog = UI::WebDialog.new("Layers Panel", false, "LayersPanel", 300, 300, 300, 200, true)
	@dialog.min_width = 200
	@html_path = File.dirname( __FILE__ ) + "/layers Panel.html"
	@dialog.set_file(@html_path)

	@dialog.add_action_callback("getModelLayers") do  |wdl, action|
		self.getModelLayers(true)
	end#callback getModelLayers
	
	def self.getModelLayers(serialize)
		# @model.start_operation("Layers Panel")
		
		serialized = @model.get_attribute("jbb_layerspanel", "serialized") #retreive string of serialized layers
		matches = serialized.to_s.scan(/(layer|group)\[(\d+)\]\=(\d+|null)/) #make an array of it
		
		matches.each do |match| 
			#match[0] : layer or group
			#match[1] : ID
			#match[2] : parent ID
			match = match.to_a
			
			if match[0].to_s == "layer" #if layer
				@layers.each{|layer| 
					if layer.get_attribute("jbb_layerspanel", "ID").to_i == match[1].to_i #if layer's dict ID == match ID
						#puts layer.name
						locked = 'false'
						if layer.get_attribute("jbb_layerspanel", "lock").to_i == 1
							locked = 'true'
						end#if
						# puts locked
						addLayerFromRuby = "addLayerFromRuby('#{layer.name}', '#{match[1]}', '#{match[2]}', false, '#{locked}');"
						@dialog.execute_script(addLayerFromRuby)
						break
					end#if
				}
			else #if group
				groupName = @model.get_attribute("jbb_layerspanel_groups", match[1])
				# puts groupName
				addGroup = "addGroup('#{groupName}', '#{match[1]}', '#{match[2]}');"
				@dialog.execute_script(addGroup)
			end#if
		end#each
		
		#Hide/show groups
		if @model.pages.selected_page == nil
			dict = @model
		else
			dict = @model.pages.selected_page
		end#if
		if dict.attribute_dictionaries["jbb_layerspanel_hiddenGroups"]
			dict.attribute_dictionaries["jbb_layerspanel_hiddenGroups"].each { | groupId, value |
				if value == 1
					hideGroupFromRuby = "hideGroupFromRuby('#{groupId}');"
					@dialog.execute_script(hideGroupFromRuby)
				elsif value == 2
					hideGroupFromRuby = "hideGroupFromRuby('#{groupId}', true);"
					@dialog.execute_script(hideGroupFromRuby)
				end#if
			}
		end#if
		
		#Hide/show layers, add missing layers
		@layers.each{|layer|
			# puts layer.name
			if layer != @layers[0]
				if layer.get_attribute("jbb_layerspanel", "ID") == nil 
					# puts 'attribute'
					self.initializeLayerDictID
					self.IdLayer(layer)
				end#if
				
				layerId = layer.get_attribute("jbb_layerspanel", "ID")
				addLayerFromRuby = "addLayerFromRuby('#{layer.name}', '#{layerId}', undefined, false);"
				@dialog.execute_script(addLayerFromRuby)
				if layer.visible?
					showLayerFromRuby = "showLayerFromRuby('#{layerId}');"
					@dialog.execute_script(showLayerFromRuby)
				elsif dict.get_attribute("jbb_layerspanel_hiddenByGroupLayers", layerId) == 1
					hideByGroupFromRuby = "hideByGroupFromRuby('#{layerId}');"
					@dialog.execute_script(hideByGroupFromRuby)
				else
					hideLayerFromRuby = "hideLayerFromRuby('#{layerId}');"
					@dialog.execute_script(hideLayerFromRuby)
				end#if
				
				if layer.get_attribute("jbb_layerspanel", "observer") != 1
					@model.start_operation("Add layer observer", false, false, true)
					layer.add_observer(JBB_LP_EntityObserver.new)
					layer.set_attribute("jbb_layerspanel", "observer", 1)
					# puts 'observer ' + layer.name
					@model.commit_operation 
				end#if
			else
			end#if
		}
		
		# @model.commit_operation
	end#def

	@dialog.add_action_callback("getCollapsedGroups") do  |wdl, action|
		self.getCollapsedGroups()
	end#callback getModelLayers

	def self.getCollapsedGroups()
		serialized = @model.get_attribute("jbb_layerspanel", "serialized") #retreive string of serialized layers
		matches = serialized.to_s.scan(/(layer|group)\[(\d+)\]\=(\d+|null)/) #make an array of it
		
		matches.each do |match| #Group collapsing/expanding
			#match[0] : layer or group
			#match[1] : ID
			#match[2] : parent ID
			match = match.to_a
			
			if match[0].to_s == "group" #if group
				# puts match[1]
				if @model.pages.selected_page == nil
					dict = @model
				else
					dict = @model.pages.selected_page
				end#if
				
				if dict.get_attribute("jbb_layerspanel_collapseGroups", match[1]).to_i == 1
					# puts match[1]
					collapseFromRuby = "collapseFromRuby('#{match[1]}');"
					@dialog.execute_script(collapseFromRuby)
				end#if
			end#if
		end#each
	end#def

	@dialog.add_action_callback("getActiveLayer") do  |wdl, action|
		self.getActiveLayer()
	end#callback getModelLayers

	def self.getActiveLayer()
		if @model.active_layer == @layers[0]
			activeLayer = 0
		else
			activeLayer = @model.active_layer.get_attribute("jbb_layerspanel", "ID")
		end#if
			setActiveLayerFromRuby = "setActiveLayerFromRuby('#{activeLayer}');"
			@dialog.execute_script(setActiveLayerFromRuby)
	end#def

	@dialog.add_action_callback("setActiveLayerFromJS") do  |wdl, layerId|
		# puts layerId
		@layers.each{|layer| 
			if layer.get_attribute("jbb_layerspanel", "ID").to_i == layerId.to_i
				@model.active_layer = layer
				
				showLayerFromRuby = "showLayerFromRuby('#{layerId}');"
				@dialog.execute_script(showLayerFromRuby)
				break
			end#if
		}
	end#callback getModelLayers

	@dialog.add_action_callback("addLayerFromJS") do
		layer = @layers.add @layers.unique_name
	end#callback addLayerFromJS

	@dialog.add_action_callback("addHiddenLayerFromJS") do
		@model.start_operation("Add layer")
		layer = @layers.add @layers.unique_name
		layer.page_behavior=(LAYER_HIDDEN_BY_DEFAULT | LAYER_IS_HIDDEN_ON_NEW_PAGES)
		@model.commit_operation
	end#callback addLayerFromJS

	@dialog.add_action_callback("renameLayerFromJS") do |wdl, layerNameS|
		@model.start_operation("Rename layer")
		hashStringLayerNames = eval( layerNameS.inspect.gsub(':','=>') ) #Convert Json string to Hash string
		hashLayerNames = eval(hashStringLayerNames) #Convert hash string to proper hash object
		layerId = hashLayerNames['layerID']
		newLayerName = hashLayerNames['newLayerName']
		@layers.each{|layer| 
			if layer.get_attribute("jbb_layerspanel", "ID").to_i == layerId.to_i
				layer.name = newLayerName
				break
			end#if
		}
		UI.refresh_inspectors
		@model.commit_operation
	end#callback renameLayerFromJS

	@dialog.add_action_callback("checkLayerForContent") do |wdl, layerId|
		# puts layerId
		@layers.each{|layer| 
			if layer.get_attribute("jbb_layerspanel", "ID").to_i == layerId.to_i
				layerHasContent = false
				
				allents=[]
				@model.entities.each{|e|allents<<e if e.valid? and e.respond_to?(:layer)and e.layer==layer}
				@model.definitions.each{|d|d.entities.each{|e|allents<<e if e.valid? and e.respond_to?(:layer)and e.layer==layer}}
				if allents != [] #Layer has content
					layerHasContent = true
				end#if
				
				checkLayerForContent = "checkLayerForContent('#{layerId}', '#{layerHasContent}');"
				@dialog.execute_script(checkLayerForContent)
				break
			end#if
		}
	end#callback

	@dialog.add_action_callback("lockFromJS") do |wdl, layerId|
		@layers.each{|layer| 
			if layer.get_attribute("jbb_layerspanel", "ID").to_i == layerId.to_i
				layer.set_attribute("jbb_layerspanel", "lock", 1)
				# puts layer.name + " locked"
				break
			end#if
		}
	end#callback

	@dialog.add_action_callback("unlockFromJS") do |wdl, layerId|
		@layers.each{|layer| 
			if layer.get_attribute("jbb_layerspanel", "ID").to_i == layerId.to_i
				layer.set_attribute("jbb_layerspanel", "lock", 0)
				# puts layer.name + " unlocked"
				break
			end#if
		}
	end#callback

	@dialog.add_action_callback("deleteLayerFromJS") do |wdl, layerId|
		@layers.each{|layer| 
			if layer.get_attribute("jbb_layerspanel", "ID").to_i == layerId.to_i
				self.deleteLayer(layer)
				break
			end#if
		}
	end#callback deleteLayerFromJS

	@dialog.add_action_callback("deleteLayer&GeomFromJS") do |wdl, layerId|
		@layers.each{|layer| 
			if layer.get_attribute("jbb_layerspanel", "ID").to_i == layerId.to_i
				self.deleteLayer(layer, true)
				break
			end#if
		}
	end#callback deleteLayerFromJS

	@dialog.add_action_callback("hideLayerFromJS") do |wdl, layerId|
		@model.start_operation("Hide layer")
		# puts layerId
		@layers.each{|layer| 
			if layer.get_attribute("jbb_layerspanel", "ID").to_i == layerId.to_i
				layer.visible = false
				break
			end#if
		}
		@model.commit_operation
	end#callback hideLayerFromJS

	@dialog.add_action_callback("showLayerFromJS") do |wdl, layerId|
		@model.start_operation("Unhide layer")
		# puts layerId
		@layers.each{|layer| 
			if layer.get_attribute("jbb_layerspanel", "ID").to_i == layerId.to_i
				layer.visible = true
				break
			end#if
		}
		@model.commit_operation
	end#callback showLayerFromJS

	def self.unHideByGroup(layerId)
		# @model.start_operation("Unhide layer", false, false, true)
		if @model.pages.selected_page == nil
			dict = @model
		else
			dict = @model.pages.selected_page
		end#if
		dict.set_attribute("jbb_layerspanel_hiddenByGroupLayers", layerId, 0)
		# puts layer.name + " unhidden by group"
		# @model.commit_operation
	end#def

	@dialog.add_action_callback("hideByGroup") do |wdl, layerId|
		@model.start_operation("Hide layer", false, false, true)
		# puts layerId
		if @model.pages.selected_page == nil
			dict = @model
		else
			dict = @model.pages.selected_page
		end#if
		dict.set_attribute("jbb_layerspanel_hiddenByGroupLayers", layerId, 1)
		@model.commit_operation
	end#callback hideByGroup

	@dialog.add_action_callback("hideGroup") do |wdl, groupId|
		# puts "Hide group " + groupId
		@model.start_operation("Hide group layer")
		if @model.pages.selected_page == nil
			dict = @model
		else
			dict = @model.pages.selected_page
			# puts 'hide - current page : ' + @model.pages.selected_page.name
		end#if
		dict.set_attribute("jbb_layerspanel_hiddenGroups", groupId, 1)
		# puts "hide group"
		@model.commit_operation
	end#callback hideGroup

	@dialog.add_action_callback("hideGroupByGroup") do |wdl, groupId|
		# puts "HideByGroup group " + groupId
		@model.start_operation("Hide group layer", false, false, true)
		if @model.pages.selected_page == nil
			dict = @model
		else
			dict = @model.pages.selected_page
			# puts 'hidebygroup - current page : ' + @model.pages.selected_page.name
		end#if
		dict.set_attribute("jbb_layerspanel_hiddenGroups", groupId, 2)
		@model.commit_operation
	end#callback hideGroup

	@dialog.add_action_callback("unHideGroup") do |wdl, groupId|
		# puts "Hide group " + groupId
		@model.start_operation("Unhide group layer")
		if @model.pages.selected_page == nil
			dict = @model
		else
			dict = @model.pages.selected_page
			# puts 'unhide - current page : ' + @model.pages.selected_page.name
		end#if
		dict.set_attribute("jbb_layerspanel_hiddenGroups", groupId, 0)
		# puts "unhide group"
		@model.commit_operation
	end#callback hideGroup

	@dialog.add_action_callback("purgeLayersFromJS") do |wdl, act|
		@model.start_operation("Purge unused layers")
		group=@model.entities.add_group()
		group.layer=nil
		gents=group.entities ### temporarily use locked layers
		temp=gents.add_group()
		temp.layer=nil
		@layers.each{|layer|
			if layer.get_attribute("jbb_layerspanel", "lock").to_i == 1
				tc=temp.copy
				tc.layer=layer
			end#if
		}
		@layers.purge_unused ### purge layer from browser
		if group.deleted? == false
			group.erase! ### erase! the temporary layer user, use set as was.
		end#if
		@model.commit_operation
	end#callback purgeLayersFromJS

	@dialog.add_action_callback("getLayerDictID") do |wdl, act|
		# @model.start_operation("Add group layer", false, false, true)
		JBB_LayersPanel.initializeLayerDictID
		sendLayerDictID = "receiveLayerDictID('#{@layerDictID}');"
		@dialog.execute_script(sendLayerDictID)
		self.incLayerDictID
		# @model.commit_operation
	end#callback getLayerDictID

	@dialog.add_action_callback("addGroupStart") do |wdl, groupName|
		@model.start_operation("Add group layer")
		self.initializeLayerDictID
		# puts groupName
		# puts @layerDictID
		@model.set_attribute("jbb_layerspanel_groups", @layerDictID, groupName) #Store group's name with ID
	end#callback addGroup

	@dialog.add_action_callback("addGroupEnd") do |wdl, groupName|
		# @dialog.execute_script("storeSerialize();")
		@model.commit_operation
	end#callback addGroup

	@dialog.add_action_callback("renameGroup") do |wdl, renameGroup|
		@model.start_operation("Rename group layer")
		hashStringGroup = eval( renameGroup.inspect.gsub(':','=>') ) #Convert Json string to Hash string
		hashGroup = eval(hashStringGroup) #Convert hash string to proper hash object
		groupId = hashGroup['groupID']
		# puts groupId
		newGroupName = hashGroup['newGroupName']
		# puts newGroupName
		@model.set_attribute("jbb_layerspanel_groups", groupId, newGroupName) #Store new group's name from ID
		@model.commit_operation
	end#callback renameLayerFromJS

	@dialog.add_action_callback("collapseGroup") do |wdl, groupId|
		# puts "collapse " + groupId
		@model.start_operation("Collapse group layer")
		if @model.pages.selected_page == nil
			dict = @model
		else
			dict = @model.pages.selected_page
		end#if
		dict.set_attribute("jbb_layerspanel_collapseGroups", groupId, 1)
		@model.commit_operation
	end#callback renameLayerFromJS

	@dialog.add_action_callback("expandGroup") do |wdl, groupId|
		# puts "expand " + groupId
		@model.start_operation("Expand group layer")
		if @model.pages.selected_page == nil
			dict = @model
		else
			dict = @model.pages.selected_page
		end#if
		dict.set_attribute("jbb_layerspanel_collapseGroups", groupId, 0)
		@model.commit_operation
	end#callback renameLayerFromJS

	@dialog.add_action_callback("storeSerialize") do |wdl, serialized|
		@model.start_operation("Store Serialize", false, false, true)
		@model.set_attribute("jbb_layerspanel", "serialized", serialized) #Store serialized in model attribute dict
		# puts serialized
		@model.commit_operation
	end#callback storeSerialize

	@dialog.add_action_callback("sortItem") do |wdl, serialized|
		@model.start_operation("Sort layer/group")
		@dialog.execute_script("storeSerialize();")
		@model.commit_operation
	end#callback 

	@dialog.add_action_callback("iframeTrack") do
		su = Sketchup.app_name
		suversion = Sketchup.version
		lang = Sketchup.get_locale
		iframeTrack = "iframeTrack('#{@lpversion}', '#{su}', '#{suversion}', '#{lang}');"
		@dialog.execute_script(iframeTrack)
	end#callback

	@dialog.add_action_callback("undo") do
		Sketchup.send_action("editUndo:")
	end#callback 

	@dialog.add_action_callback("redo") do
		Sketchup.send_action("editRedo:")
	end#callback

	@dialog.set_on_close do
		# @dialog.execute_script("storeSerialize();")
	end
	
  
  
	### @layersOBSERVER ### ------------------------------------------------------
	
	# Layers observer
	class JBB_LP_layersObserver < Sketchup::LayersObserver
	
		def onLayerAdded(layers, layer)
			JBB_LayersPanel.model.start_operation("Add layer", false, false, true)
			JBB_LayersPanel.initializeLayerDictID
			JBB_LayersPanel.IdLayer(layer)
			layerIdForJS = layer.get_attribute("jbb_layerspanel", "ID")
			addLayerFromRuby = "addLayerFromRuby('#{layer.name}', '#{layerIdForJS}');"
			JBB_LayersPanel.dialog.execute_script(addLayerFromRuby)
			showLayerFromRuby = "showLayerFromRuby('#{layerIdForJS}');"
			JBB_LayersPanel.dialog.execute_script(showLayerFromRuby)
			
			# Attach the entity observer.
			JBB_LayersPanel.model.start_operation("Add layer observer", false, false, true)
			layer.add_observer(JBB_LP_EntityObserver.new)
			layer.set_attribute("jbb_layerspanel", "observer", 1)
			# puts 'observer ' + layer.name
			JBB_LayersPanel.model.commit_operation 
			
			# JBB_LayersPanel.dialog.execute_script("storeSerialize();")
			JBB_LayersPanel.model.commit_operation
		end#onLayerAdded
		
		def onLayerRemoved(layers, layer)
			layerId = layer.get_attribute("jbb_layerspanel", "ID")
			deleteLayerFromRuby = "deleteLayerFromRuby('#{layerId}');"
			JBB_LayersPanel.dialog.execute_script(deleteLayerFromRuby)
			if JBB_LayersPanel.allowSerialize == true
				JBB_LayersPanel.dialog.execute_script("storeSerialize();")
			end#if
		end#onLayerRemoved
		
		
		def onCurrentLayerChanged(layers, layer)
			JBB_LayersPanel.dialog.execute_script('getActiveLayer();')
		end#onLayerRemoved
		
	end#JBB_LP_layersObserver
	
	# Attach the observer.
	@layers.add_observer(JBB_LP_layersObserver.new)
	
  
  
	### ENTITYOBSERVER ### ------------------------------------------------------
	#Watches for layers to be hidden or renamed (Which layers observer doesn't support)
	
	class JBB_LP_EntityObserver < Sketchup::EntityObserver
		def onChangeEntity(layer)
			if layer.deleted? == false
				# puts 'onchangeentity ' + layer.name
				if layer.get_attribute("jbb_layerspanel", "ID") != nil #Verify entity exists (onChangeEntity mistrigger)
					# puts layer.name
					if layer == JBB_LayersPanel.layers[0]
						layerId = 0
					else
						layerId = layer.get_attribute("jbb_layerspanel", "ID")
					end#if
					
					if layer.visible?
						showLayerFromRuby = "showLayerFromRuby('#{layerId}');"
						JBB_LayersPanel.dialog.execute_script(showLayerFromRuby)
						JBB_LayersPanel.unHideByGroup(layerId)
					else
						hideLayerFromRuby = "hideLayerFromRuby('#{layerId}');"
						JBB_LayersPanel.dialog.execute_script(hideLayerFromRuby)
					end#if
					
					renameLayerFromRuby = "renameLayerFromRuby('#{layerId}', '#{layer.name}');"
					JBB_LayersPanel.dialog.execute_script(renameLayerFromRuby)
					
					if	JBB_LayersPanel.model.pages.selected_page != nil
						JBB_LayersPanel.model.pages.selected_page.update(32) #Update page's layers state
						# puts "update " + JBB_LayersPanel.model.pages.selected_page.name
					end#if
				end#if
			end#if
		end#def
	end#class
	
	# Attach the observer to layer0
	@layers[0].add_observer(JBB_LP_EntityObserver.new)
	
  
  
	### APPOBSERVER ### ------------------------------------------------------
	
	class JBB_LP_AppObserver < Sketchup::AppObserver

		def onNewModel(newModel)
			JBB_LayersPanel.model = newModel
			JBB_LayersPanel.layers = newModel.layers
			
			JBB_LayersPanel.layerDictID = nil
			
			JBB_LayersPanel.model.add_observer(JBB_LP_ModelObserver.new)
			JBB_LayersPanel.model.pages.add_observer(JBB_LP_PagesObserver.new)
			JBB_LayersPanel.layers.add_observer(JBB_LP_layersObserver.new)
			
			JBB_LayersPanel.layers.each{|layer| 
				layer.add_observer(JBB_LP_EntityObserver.new)
			}
			
			JBB_LayersPanel.dialog.execute_script("reloadDialog();")
		end#def

		def onOpenModel(newModel)
			JBB_LayersPanel.model = newModel
			JBB_LayersPanel.layers = newModel.layers
			
			JBB_LayersPanel.layerDictID = nil
			
			JBB_LayersPanel.model.add_observer(JBB_LP_ModelObserver.new)
			JBB_LayersPanel.model.pages.add_observer(JBB_LP_PagesObserver.new)
			JBB_LayersPanel.layers.add_observer(JBB_LP_layersObserver.new)
			
			JBB_LayersPanel.layers.each{|layer| 
				layer.add_observer(JBB_LP_EntityObserver.new)
			}
			
			JBB_LayersPanel.dialog.execute_script("reloadDialog();")
		end#def

	end#class

	# Attach the observer
	Sketchup.add_observer(JBB_LP_AppObserver.new)
	
  
  
	### MODELOBSERVER ### ------------------------------------------------------
	
	class JBB_LP_ModelObserver < Sketchup::ModelObserver
		def onTransactionUndo(model)
			JBB_LayersPanel.dialog.execute_script("emptyOl();")
			JBB_LayersPanel.getModelLayers(false)
			JBB_LayersPanel.getActiveLayer()
			JBB_LayersPanel.getCollapsedGroups()
		end#def
		def onTransactionRedo(model)
			JBB_LayersPanel.dialog.execute_script("emptyOl();")
			JBB_LayersPanel.getModelLayers(false)
			JBB_LayersPanel.getActiveLayer()
			JBB_LayersPanel.getCollapsedGroups()
		end#def
	end#class

	# Attach the observer
	@model.add_observer(JBB_LP_ModelObserver.new)
	
  
  
	### PAGESOBSERVER ### ------------------------------------------------------
	
	class JBB_LP_PagesObserver < Sketchup::PagesObserver
		def onContentsModified(pages)
			if JBB_LayersPanel.check == 1
				JBB_LayersPanel.previousPageDict = JBB_LayersPanel.model.pages.selected_page.attribute_dictionary "jbb_layerspanel_collapseGroups", true
				JBB_LayersPanel.previousPageDict2 = JBB_LayersPanel.model.pages.selected_page.attribute_dictionary "jbb_layerspanel_hiddenGroups", true
				JBB_LayersPanel.previousPageDict3 = JBB_LayersPanel.model.pages.selected_page.attribute_dictionary "jbb_layerspanel_hiddenByGroupLayers", true
				JBB_LayersPanel.dialog.execute_script("emptyOl();")
				JBB_LayersPanel.getModelLayers(false)
				JBB_LayersPanel.getActiveLayer()
				JBB_LayersPanel.getCollapsedGroups()
				JBB_LayersPanel.check = 0
			else
				JBB_LayersPanel.check = 1
			end#if
			# puts JBB_LayersPanel.check
			# puts JBB_LayersPanel.model.pages.selected_page.name
		end#def
		def onElementAdded(pages, page)
			if JBB_LayersPanel.previousPageDict == nil
				dict = JBB_LayersPanel.model.attribute_dictionary "jbb_layerspanel_collapseGroups", true
			else
				dict = JBB_LayersPanel.previousPageDict
			end#if
			dict.each { | key, value |
			   page.set_attribute("jbb_layerspanel_collapseGroups", key, value)
			}
			
			if JBB_LayersPanel.previousPageDict2 == nil
				dict2 = JBB_LayersPanel.model.attribute_dictionary "jbb_layerspanel_hiddenGroups", true
			else
				dict2 = JBB_LayersPanel.previousPageDict2
			end#if
			dict2.each { | key, value |
			   page.set_attribute("jbb_layerspanel_hiddenGroups", key, value)
			}
			
			if JBB_LayersPanel.previousPageDict3 == nil
				dict3 = JBB_LayersPanel.model.attribute_dictionary "jbb_layerspanel_hiddenByGroupLayers", true
			else
				dict3 = JBB_LayersPanel.previousPageDict3
			end#if
			dict3.each { | key, value |
			   page.set_attribute("jbb_layerspanel_hiddenByGroupLayers", key, value)
			}
			# puts "added " + page.name
			JBB_LayersPanel.dialog.execute_script("reloadDialog();")
			JBB_LayersPanel.check = 1
		end#def
	end#class

	# Attach the observer
	@model.pages.add_observer(JBB_LP_PagesObserver.new)
  
  
  
	### MENU & TOOLBARS ### ------------------------------------------------------
	
	def self.toggle_layerspanel_dlg
		if @dialog.visible?
			@dialog.close
			false
		else
			@dialog.show()
			self.make_toolwindow_frame("Layers Panel")
			true
		end#if
	end#def
	
	# Thomthom's snippet :
	# http://sketchucation.com/forums/viewtopic.php?p=280331#p280331
	def self.make_toolwindow_frame(window_title)
		# Retrieves the window handle to the active window attached to the calling
		# thread's message queue. 
		hwnd = GetActiveWindow.call
		return nil if hwnd.nil?

		# Verify window text as extra security to ensure it's the correct window.
		buf_len = GetWindowTextLength.call(hwnd)
		return nil if buf_len == 0

		str = ' ' * (buf_len + 1)
		result = GetWindowText.call(hwnd, str, str.length)
		return nil if result == 0

		return nil unless str.strip == window_title.strip

		# Set frame to Toolwindow
		style = GetWindowLong.call(hwnd, GWL_EXSTYLE)
		return nil if style == 0

		result = SetWindowLong.call(hwnd, GWL_EXSTYLE, style | WS_EX_TOOLWINDOW)
		return nil if result == 0

		# Remove and disable minimze and maximize
		# http://support.microsoft.com/kb/137033
		style = GetWindowLong.call(hwnd, GWL_STYLE)
		return nil if style == 0

		style = style & ~WS_MINIMIZEBOX
		style = style & ~WS_MAXIMIZEBOX
		result = SetWindowLong.call(hwnd, GWL_STYLE,  style)
		return nil if result == 0

		# Refresh the window frame
		result = SetWindowPos.call(hwnd, 0, 0, 0, 0, 0, SWP_FRAMECHANGED|SWP_NOSIZE|SWP_NOMOVE)
		result != 0
	end
	
	def self.layerspanel_dlg_validation_proc
		if @dialog && @dialog.visible?
			MF_CHECKED
		else
			MF_UNCHECKED
		end#if
	end#def
  
	unless file_loaded?( __FILE__ )
		# Commands
		cmd = UI::Command.new( 'Layers Panel' ) { self.toggle_layerspanel_dlg }
		cmd.status_bar_text = 'Show or hide the Layers Panel.'
		cmd.small_icon = "lp_16.png"
		cmd.large_icon = "lp_24.png"
		cmd.tooltip = 'Layers Panel'
		cmd.set_validation_proc { self.layerspanel_dlg_validation_proc }
		cmd_toggle_layerspanel_dlg = cmd

		window_menu = UI.menu("Window")
		window_menu.add_item( cmd_toggle_layerspanel_dlg )
		
		layerspanel_tb = UI::Toolbar.new "Layers Panel"
		layerspanel_tb.add_item cmd_toggle_layerspanel_dlg
		layerspanel_tb.show
	end 
	
end#module

#-----------------------------------------------------------------------------
file_loaded( File.basename(__FILE__) )
